@@ -212,6 +212,96 @@ class mgrepvi(mgrepv):
     """
     flags = re.IGNORECASE
 
+class sgrep(TextOp):
+    r"""Switch grep
+
+    This works like :class:`textops.mgrep` except that it returns a list of lists.
+    ``sgrep`` dispatches lines matching a pattern to the list corresponding to the pattern order.
+    If a line matches the third pattern, it will be dispatched to the third returned list.
+    If N patterns are given to search, it will return N+1 lists, where the last list will be filled
+    of lines that does not match any pattern in the given patterns list.
+    The patterns list order is important : only the first
+    matching pattern will be taken in account.
+    One can consider that ``sgrep`` works like a **switch()** :
+    it will do for each line a kind of ::
+
+        if pattern1 matches:
+            put line in list1
+        elif pattern2 matches:
+            put line in list2
+        elif patternN matches:
+            put line in listN
+        else:
+            put line in listN+1
+
+    Args:
+        patterns (list): a list of patterns to search.
+        key (int or str): test only one column or one key (optional)
+
+    Returns:
+        list: a list of lists (nb patterns + 1)
+
+    Examples:
+        >>> logs = '''
+        ... error 1
+        ... warning 1
+        ... warning 2
+        ... info 1
+        ... error 2
+        ... info 2
+        ... '''
+        >>> t = logs | sgrep(('^err','^warn'))
+        >>> print t                                         #doctest: +NORMALIZE_WHITESPACE
+        [['error 1', 'error 2'], ['warning 1', 'warning 2'], ['', 'info 1', 'info 2']]
+    """
+    flags = 0
+    reverse = False
+    @classmethod
+    def op(cls,text,patterns,key = None, *args,**kwargs):
+        patterns = [ re.compile(pattern,cls.flags) if isinstance(pattern,basestring) else pattern for pattern in patterns ]
+        lst = [ [] for i in xrange(len(patterns)+1) ] # surtout pas faire [] * (len(patterns)+1)
+        for line in cls._tolist(text):
+            for i,regex in enumerate(patterns):
+                try:
+                    if isinstance(line,basestring):
+                        if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
+                            lst[i].append(line)
+                            break
+                    elif key is None:
+                        if bool(regex.search(stru(line))) != cls.reverse:  # kind of XOR with cls.reverse
+                            lst[i].append(line)
+                            break
+                    else:
+                        if bool(regex.search(stru(line[key]))) != cls.reverse:  # kind of XOR with cls.reverse
+                            lst[i].append(line)
+                            break
+                except (ValueError, TypeError, IndexError, KeyError):
+                    pass
+            else:
+                lst[-1].append(line)
+        return lst
+
+class sgrepi(sgrep):
+    r"""Switch grep case insensitive
+
+    This works like :class:`textops.sgrep` but is case insensitive
+    """
+    flags = re.IGNORECASE
+
+class sgrepv(sgrep):
+    r"""Switch grep reversed
+
+    This works like :class:`textops.sgrep` except that it tests that patterns DOES NOT match the line.
+    """
+    reverse = True
+
+class sgrepvi(sgrepv):
+    r"""Switch grep reversed case insensitive
+
+    This works like :class:`textops.sgrepv` but is case insensitive
+    """
+    flags = re.IGNORECASE
+
 class parseg(TextOp):
     r"""Find all occurrences of one pattern, return MatchObject groupdict
 
@@ -336,7 +426,11 @@ class parsekv(TextOp):
         pattern (str): a regular expression string.
         key_name (str): The key name to optain the value that will be the key of the groupdict
             ('key' by default)
-        key_update (callable): function to convert/normalize the calculated key
+        key_update (callable): function to convert/normalize the calculated key.
+            If ``None``, the keys is normalized.
+            If not ``None`` but not callable ,the key is unchanged.
+        val_name (str): instead of storing the groupdict, on can choose to select
+            the value at the key ``val_name`. (by default, None : means the whole groupdict)
 
     Returns:
         dict: A dict of MatchObject groupdicts
@@ -353,10 +447,19 @@ class parsekv(TextOp):
         {'FIRST NAME': {'item': 'first name', 'val': 'Eric'},
         'NAME': {'item': 'name', 'val': 'Lapouyade'},
         'COUNTRY': {'item': 'country', 'val': 'France'}}
+        >>> s | parsekv(r'(?P<key>.*):\s*(?P<val>.*)',key_update=0)         #doctest: +NORMALIZE_WHITESPACE
+        {'first name': {'val': 'Eric', 'key': 'first name'},
+        'name': {'val': 'Lapouyade', 'key': 'name'},
+        'country': {'val': 'France', 'key': 'country'}}
+        >>> s | parsekv(r'(?P<key>.*):\s*(?P<val>.*)',val_name='val')         #doctest: +NORMALIZE_WHITESPACE
+        {'country': 'France', 'first_name': 'Eric', 'name': 'Lapouyade'}
     """
     ignore_case = False
+    val_name = None
     @classmethod
-    def op(cls,text, pattern, key_name = 'key', key_update = None, *args,**kwargs):
+    def op(cls,text, pattern, key_name = 'key', key_update = None, val_name = None, *args,**kwargs):
+        if val_name is None:
+            val_name = cls.val_name
         if isinstance(pattern,basestring):
             pattern = re.compile(pattern, re.I if cls.ignore_case else 0)
         out = {}
@@ -368,9 +471,11 @@ class parsekv(TextOp):
                 if key:
                     if key_update is None:
                         key_norm = index_normalize(key)
-                    else:
+                    elif callable(key_update):
                         key_norm = key_update(key)
-                    out.update({ key_norm : dct })
+                    else:
+                        key_norm = key
+                    out.update({ key_norm : dct if val_name is None else dct[val_name]})
         return out
 
 class parsekvi(parsekv):
@@ -382,7 +487,11 @@ class parsekvi(parsekv):
         pattern (str): a regular expression string (case insensitive).
         key_name (str): The key name to optain the value that will be the key of the groupdict
             ('key' by default)
-        key_update (callable): function to convert/normalize the calculated key
+        key_update (callable): function to convert/normalize the calculated key.
+            If ``None``, the keys is normalized.
+            If not ``None`` but not callable ,the key is unchanged.
+        val_name (str): instead of storing the groupdict, on can choose to select
+            the value at the key ``val_name`. (by default, None : means the whole groupdict)
 
     Returns:
         dict: A dict of MatchObject groupdicts
@@ -396,6 +505,60 @@ class parsekvi(parsekv):
     """
     ignore_case = True
 
+class keyval(parsekv):
+    r"""Return a dictionnay where keys and values are taken from the pattern specify
+
+    It is a shortcut for :class:`textops.parsekv` with val_name='val'
+
+    Args:
+        pattern (str): a regular expression string.
+        key_name (str): The key name to optain the value that will be the key of the groupdict
+            ('key' by default)
+        key_update (callable): function to convert/normalize the calculated key.
+            If ``None``, the keys is normalized.
+            If not ``None`` but not callable ,the key is unchanged.
+        val_name (str): instead of storing the groupdict, on can choose to select
+            the value at the key ``val_name`. (by default, None means 'val')
+
+    Returns:
+        dict: A dict of key:val from the matched pattern groupdict
+
+    Examples:
+        >>> s = '''name: Lapouyade
+        ... first name: Eric
+        ... country: France'''
+        >>> s | keyval(r'(?P<key>.*):\s*(?P<val>.*)')         #doctest: +NORMALIZE_WHITESPACE
+        {'country': 'France', 'first_name': 'Eric', 'name': 'Lapouyade'}
+    """
+    val_name = 'val'
+
+class keyvali(keyval):
+    r"""Return a dictionnay where keys and values are taken from the pattern specify
+
+    It works a little like :class:`textops.keyval` except that the pattern is case insensitive.
+
+    Args:
+        pattern (str): a regular expression string (case insensitive).
+        key_name (str): The key name to optain the value that will be the key of the groupdict
+            ('key' by default)
+        key_update (callable): function to convert/normalize the calculated key.
+            If ``None``, the keys is normalized.
+            If not ``None`` but not callable ,the key is unchanged.
+        val_name (str): instead of storing the groupdict, on can choose to select
+            the value at the key ``val_name`. (by default, None means 'val')
+
+    Returns:
+        dict: A dict of key:val from the matched pattern groupdict
+
+    Examples:
+        >>> s = '''name IS Lapouyade
+        ... first name IS Eric
+        ... country IS France'''
+        >>> s | keyvali(r'(?P<key>.*) is (?P<val>.*)')         #doctest: +NORMALIZE_WHITESPACE
+        {'country': 'France', 'first_name': 'Eric', 'name': 'Lapouyade'}
+    """
+    ignore_case = True
+
 class find_pattern(TextOp):
     r"""Fast pattern search
 
